% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gather_emmeans_draws.R
\name{gather_emmeans_draws}
\alias{gather_emmeans_draws}
\title{Extract a tidy data frame of draws of posterior distributions of "estimated marginal means" (emmeans/lsmeans) from
a Bayesian model fit.}
\usage{
gather_emmeans_draws(object, value = ".value")
}
\arguments{
\item{object}{An \code{emmGrid} object such as returned by
\code{\link[emmeans]{ref_grid}} or \code{\link[emmeans]{emmeans}}.}

\item{value}{The name of the output column to use to contain the values of draws. Defaults to \code{".value"}.}
}
\value{
A tidy data frame of draws. The columns of the reference grid are returned as-is, with an
additional column called \code{.value} (by default) containing marginal draws. The resulting data
frame is grouped by the columns from the reference grid to make use of summary functions like
\code{\link{point_interval}} straightforward.
}
\description{
Extract draws from the result of a call to \code{\link[emmeans]{emmeans}} (formerly \code{lsmeans})
or \code{\link[emmeans]{ref_grid}} applied to a Bayesian model.
}
\details{
\code{\link[emmeans]{emmeans}} provides a convenient syntax for generating draws from "estimated marginal means" from a model,
and can be applied to various Bayesian models, like \code{\link[rstanarm]{stanreg-objects}} and
\code{\link[MCMCglmm]{MCMCglmm}}. Given a \code{\link[emmeans]{ref_grid}} object as returned by functions like
\code{\link[emmeans]{ref_grid}} or \code{\link[emmeans]{emmeans}} applied to a Bayesian model,
\code{gather_emmeans_draws} returns a tidy format data frame of draws from
the marginal posterior distributions generated by \code{\link[emmeans]{emmeans}}.
}
\examples{

library(dplyr)
library(magrittr)

if (
  require("rstanarm", quietly = TRUE) &&
  require("emmeans", quietly = TRUE)
) {

  # Here's an example dataset with a categorical predictor (`condition`) with several levels:
  set.seed(5)
  n = 10
  n_condition = 5
  ABC =
    data_frame(
      condition = rep(c("A","B","C","D","E"), n),
      response = rnorm(n * 5, c(0,1,2,1,-1), 0.5)
    )

  m = stan_glm(response ~ condition, data = ABC,
    # 1 chain / few iterations just so example runs quickly
    # do not use in practice
    chains = 1, iter = 500)

  # Once we've fit the model, we can use emmeans() (and functions
  # from that package) to get whatever marginal distributions we want.
  # For example, we can get marginal means by condition:
  m \%>\%
    emmeans(~ condition) \%>\%
    gather_emmeans_draws() \%>\%
    median_qi()

  # or we could get pairwise differences:
  m \%>\%
    emmeans(~ condition) \%>\%
    contrast(method = "pairwise") \%>\%
    gather_emmeans_draws() \%>\%
    median_qi()

  # see the documentation of emmeans() for more examples of types of
  # contrasts supported by that packge.
}
}
\seealso{
\code{\link[emmeans]{emmeans}}
}
\author{
Matthew Kay
}
\keyword{manip}
